@@ -1,11 +1,25 @@
 # Paranoid::Args -- Command-line argument parsing functions
 #
-# (c) 2005 - 2017, Arthur Corliss <corliss@digitalmages.com>
+# $Id: lib/Paranoid/Args.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 #
-# $Id: lib/Paranoid/Args.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+# This software is free software.  Similar to Perl, you can redistribute it
+# and/or modify it under the terms of either:
 #
-#    This software is licensed under the same terms as Perl, itself.
-#    Please see http://dev.perl.org/licenses/ for more information.
+#   a)     the GNU General Public License
+#          <https://www.gnu.org/licenses/gpl-1.0.html> as published by the
+#          Free Software Foundation <http://www.fsf.org/>; either version 1
+#          <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+#          <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+#   b)     the Artistic License 2.0
+#          <https://opensource.org/licenses/Artistic-2.0>,
+#
+# subject to the following additional term:  No trademark rights to
+# "Paranoid" have been or are conveyed under any of the above licenses.
+# However, "Paranoid" may be used fairly to describe this unmodified
+# software, in good faith, but not as a trademark.
+#
+# (c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+# (tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 #
 #####################################################################
 
@@ -26,11 +40,14 @@ use base qw(Exporter);
 use Paranoid;
 use Paranoid::Debug qw(:all);
 
-($VERSION) = ( q$Revision: 2.07 $ =~ /(\d+(?:\.\d+)+)/sm );
+($VERSION) = ( q$Revision: 2.10 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(parseArgs);
 @EXPORT_OK   = ( @EXPORT, qw(PA_DEBUG PA_VERBOSE PA_HELP PA_VERSION) );
-%EXPORT_TAGS = ( all => [@EXPORT_OK], );
+%EXPORT_TAGS = (
+    all      => [@EXPORT_OK],
+    template => [qw(PA_DEBUG PA_VERBOSE PA_HELP PA_VERSION)],
+    );
 
 # I know, this really doesn't protect the contents...
 use constant PA_DEBUG => {
@@ -207,8 +224,7 @@ sub _tLint {
     my $rv   = 1;
     my ( $oname, @at );
 
-    pdebug( 'entering w/(%s)', PDLEVEL2, $tref );
-    pIn();
+    subPreamble( PDLEVEL2, '$', $tref );
 
     # Get the option name for reporting purposes (should have been populated
     # within parseArgs below)
@@ -315,8 +331,7 @@ sub _tLint {
         }
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL2, $rv );
+    subPostamble( PDLEVEL2, '$', $rv );
 
     return $rv;
 }
@@ -336,9 +351,7 @@ sub _getArgs ($$\@) {
     my $argRef      = _getArgRef();
     my @tmp;
 
-    pdebug( 'entering w/(%s)(%s)(%s)',
-        PDLEVEL2, $option, $argTemplate, $lref );
-    pIn();
+    subPreamble( PDLEVEL2, '$$$', $option, $argTemplate, $lref );
 
     # Empty the array
     @$lref = ();
@@ -456,8 +469,7 @@ sub _getArgs ($$\@) {
         pdebug( 'sublisted arguments into: %s', PDLEVEL3, @$lref );
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL2, $rv );
+    subPostamble( PDLEVEL2, '$', $rv );
 
     return $rv;
 }
@@ -474,8 +486,7 @@ sub _storeArgs ($$\@) {
     my $argTemplate = shift;
     my $lref        = shift;
 
-    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL2, $tref, $argTemplate, $lref );
-    pIn();
+    subPreamble( PDLEVEL2, '$$$', $tref, $argTemplate, $lref );
 
     pdebug( 'adding values to %s', PDLEVEL3, $$tref{Name} );
 
@@ -534,8 +545,7 @@ sub _storeArgs ($$\@) {
         }
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: 1', PDLEVEL2 );
+    subPostamble( PDLEVEL1, '$', 1 );
 
     return 1;
 }
@@ -555,12 +565,11 @@ sub parseArgs (\@\%;\@) {
     my ( $tref, $oname, $argRef, $arg, $argTemplate );
     my ( @tmp, @oargs, $regex );
 
+    subPreamble( PDLEVEL1, '$$$', $tlref, $oref, $paref );
+
     # Validate arguments
     $paref = \@ARGV unless defined $paref;
 
-    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $tlref, $oref, $paref );
-    pIn();
-
     # Clear all internal data structures and reset flag
     clearMemory();
 
@@ -890,8 +899,7 @@ sub parseArgs (\@\%;\@) {
         $$oref{$_} = $$tref{Value};
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -906,7 +914,7 @@ Paranoid::Args - Command-line argument parsing functions
 
 =head1 VERSION
 
-$Id: lib/Paranoid/Args.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+$Id: lib/Paranoid/Args.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -931,6 +939,19 @@ when it comes to this particular task.
 The primary focus of this module is validation, with the secondary focus being
 preservation of context.
 
+=head1 IMPORT LISTS
+
+This module exports the following symbols by default:
+
+    parseArgs
+
+The following specialized import lists also exist:
+
+    List        Members
+    --------------------------------------------------------
+    template    PA_DEBUG PA_VERBOSE PA_HELP PA_VERSION
+    all         @defaults @template 
+
 =head2 VALIDATION
 
 When validating the use of options and arguments we concern ourselves
@@ -1406,8 +1427,22 @@ Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is licensed under the same terms as Perl, itself. 
-Please see http://dev.perl.org/licenses/ for more information.
+This software is free software.  Similar to Perl, you can redistribute it
+and/or modify it under the terms of either:
 
-(c) 2005 - 2017, Arthur Corliss (corliss@digitalmages.com)
+  a)     the GNU General Public License
+         <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+         Free Software Foundation <http://www.fsf.org/>; either version 1
+         <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+         <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+  b)     the Artistic License 2.0
+         <https://opensource.org/licenses/Artistic-2.0>,
+
+subject to the following additional term:  No trademark rights to
+"Paranoid" have been or are conveyed under any of the above licenses.
+However, "Paranoid" may be used fairly to describe this unmodified
+software, in good faith, but not as a trademark.
+
+(c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+(tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 
