@@ -1,11 +1,25 @@
 # Paranoid::IO::Line -- Paranoid Line-based I/O functions
 #
-# (c) 2005 - 2017, Arthur Corliss <corliss@digitalmages.com>
+# $Id: lib/Paranoid/IO/Line.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 #
-# $Id: lib/Paranoid/IO/Line.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+# This software is free software.  Similar to Perl, you can redistribute it
+# and/or modify it under the terms of either:
 #
-#    This software is licensed under the same terms as Perl, itself.
-#    Please see http://dev.perl.org/licenses/ for more information.
+#   a)     the GNU General Public License
+#          <https://www.gnu.org/licenses/gpl-1.0.html> as published by the
+#          Free Software Foundation <http://www.fsf.org/>; either version 1
+#          <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+#          <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+#   b)     the Artistic License 2.0
+#          <https://opensource.org/licenses/Artistic-2.0>,
+#
+# subject to the following additional term:  No trademark rights to
+# "Paranoid" have been or are conveyed under any of the above licenses.
+# However, "Paranoid" may be used fairly to describe this unmodified
+# software, in good faith, but not as a trademark.
+#
+# (c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+# (tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 #
 #####################################################################
 
@@ -29,7 +43,7 @@ use Paranoid::Debug qw(:all);
 use Paranoid::IO qw(:all);
 use Paranoid::Input qw(:all);
 
-($VERSION) = ( q$Revision: 2.07 $ =~ /(\d+(?:\.\d+)+)/sm );
+($VERSION) = ( q$Revision: 2.10 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(sip nlsip tailf nltailf slurp nlslurp piolClose);
 @EXPORT_OK   = ( @EXPORT, qw(PIOMAXLNSIZE) );
@@ -81,8 +95,7 @@ use constant PBF_DELETE => -1;
         my $rv   = 0;
         my ( $fh, $fpos, @fstat, @fhstat );
 
-        pdebug( 'entering w/(%s)', PDLEVEL3, $file );
-        pIn();
+        subPreamble( PDLEVEL3, '$', $file );
 
         # Check to see if we can get a valid file handle
         if ( defined( $fh = popen( $file, O_RDONLY ) ) ) {
@@ -124,8 +137,7 @@ use constant PBF_DELETE => -1;
             pdebug( 'invalid/non-existent file', PDLEVEL3 );
         }
 
-        pOut();
-        pdebug( 'leaving w/rv: %s', PDLEVEL3, $rv );
+        subPostamble( PDLEVEL3, '$', $rv );
 
         return $rv;
     }
@@ -159,8 +171,7 @@ use constant PBF_DELETE => -1;
         my $rv      = 1;
         my ( $buffer, $bflag, $in, $content, $bread, $irv, @tmp, $line );
 
-        pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $file, $aref, $doChomp );
-        pIn();
+        subPreamble( PDLEVEL1, '$\@;$$', $file, $aref, $doChomp, $noLocks );
 
         @$aref = ();
 
@@ -293,8 +304,7 @@ use constant PBF_DELETE => -1;
 
         pdebug( 'returning %s lines', PDLEVEL2, scalar @$aref );
 
-        pOut();
-        pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+        subPostamble( PDLEVEL1, '$', $rv );
 
         return $rv;
     }
@@ -330,9 +340,8 @@ sub tailf ($\@;$$$) {
     my $noLocks = shift;
     my ( $rv, $ofsb, @lines );
 
-    pdebug( 'entering w/(%s)(%s)(%s)(%s)',
-        PDLEVEL1, $file, $aref, $doChomp, $offset );
-    pIn();
+    subPreamble( PDLEVEL1, '$\@;$$$', $file, $aref, $doChomp, $offset,
+        $noLocks );
 
     @$aref = ();
 
@@ -385,8 +394,7 @@ sub tailf ($\@;$$$) {
             : sip( $file, @$aref, $doChomp );
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -419,8 +427,7 @@ sub slurp ($\@;$$) {
     my $rv      = 1;
     my @fstat;
 
-    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $file, $aref, $doChomp );
-    pIn();
+    subPreamble( PDLEVEL1, '$\@;$$', $file, $aref, $doChomp, $noLocks );
 
     # Start sipping
     $rv = sip( $file, @$aref, $doChomp, $noLocks );
@@ -440,8 +447,7 @@ sub slurp ($\@;$$) {
     # Close everything out
     piolClose($file);
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -470,7 +476,7 @@ Paranoid::IO::Line - Paranoid Line-based I/O functions
 
 =head1 VERSION
 
-$Id: lib/Paranoid/IO/Line.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+$Id: lib/Paranoid/IO/Line.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -488,6 +494,11 @@ $Id: lib/Paranoid/IO/Line.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
 
   $nlines = slurp($filename, @lines);
 
+  # Non-locking variants
+  $nlines = nlsip($filename, @lines);
+  $nlines = nltailf($filename, @lines);
+  $nlines = nlslurp($filename, @lines);
+
 =head1 DESCRIPTION
 
 This module extends and leverages L<Paranoid::IO>'s capabilities with an eye
@@ -520,6 +531,18 @@ with a record separator (new line sequence).
 As with all L<Paranoid> modules string descriptions of errors can be retrieved
 from L<Paranoid::ERROR> as they occur.
 
+=head1 IMPORT LISTS
+
+This module exports the following symbols by default:
+
+    sip nlsip tailf nltailf slurp nlslurp piolClose
+
+The following specialized import lists also exist:
+
+    List        Members
+    --------------------------------------------------------
+    all         @defaults PIOMAXLNSIZE
+
 =head1 SUBROUTINES/METHODS
 
 =head2 PIOMAXLNSIZE
@@ -684,8 +707,22 @@ Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is licensed under the same terms as Perl, itself. 
-Please see http://dev.perl.org/licenses/ for more information.
+This software is free software.  Similar to Perl, you can redistribute it
+and/or modify it under the terms of either:
 
-(c) 2005 - 2017, Arthur Corliss (corliss@digitalmages.com)
+  a)     the GNU General Public License
+         <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+         Free Software Foundation <http://www.fsf.org/>; either version 1
+         <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+         <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+  b)     the Artistic License 2.0
+         <https://opensource.org/licenses/Artistic-2.0>,
+
+subject to the following additional term:  No trademark rights to
+"Paranoid" have been or are conveyed under any of the above licenses.
+However, "Paranoid" may be used fairly to describe this unmodified
+software, in good faith, but not as a trademark.
+
+(c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+(tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 
