@@ -1,11 +1,25 @@
 # Paranoid::Module -- Paranoid Module Loading Routines
 #
-# (c) 2005 - 2017, Arthur Corliss <corliss@digitalmages.com>
+# $Id: lib/Paranoid/Module.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 #
-# $Id: lib/Paranoid/Module.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+# This software is free software.  Similar to Perl, you can redistribute it
+# and/or modify it under the terms of either:
 #
-#    This software is licensed under the same terms as Perl, itself.
-#    Please see http://dev.perl.org/licenses/ for more information.
+#   a)     the GNU General Public License
+#          <https://www.gnu.org/licenses/gpl-1.0.html> as published by the
+#          Free Software Foundation <http://www.fsf.org/>; either version 1
+#          <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+#          <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+#   b)     the Artistic License 2.0
+#          <https://opensource.org/licenses/Artistic-2.0>,
+#
+# subject to the following additional term:  No trademark rights to
+# "Paranoid" have been or are conveyed under any of the above licenses.
+# However, "Paranoid" may be used fairly to describe this unmodified
+# software, in good faith, but not as a trademark.
+#
+# (c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+# (tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 #
 #####################################################################
 
@@ -28,7 +42,7 @@ use Paranoid::Debug qw(:all);
 use Paranoid::Input;
 use Carp;
 
-($VERSION) = ( q$Revision: 2.07 $ =~ /(\d+(?:\.\d+)+)/sm );
+($VERSION) = ( q$Revision: 2.10 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(loadModule);
 @EXPORT_OK   = @EXPORT;
@@ -64,8 +78,7 @@ use Carp;
         croak 'Mandatory first argument must be a defined module name'
             unless defined $module;
 
-        pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $module, @args );
-        pIn();
+        subPreamble( PDLEVEL1, '$;@', $module, @args );
 
         # Check to see if module has been loaded already
         unless ( exists $modules{$module} ) {
@@ -124,8 +137,7 @@ EOF
             $rv = $imports{$cm};
         }
 
-        pOut();
-        pdebug( 'leaving w/rv: %s', PDLEVEL1, $modules{$module} );
+        subPostamble( PDLEVEL1, '$', $modules{$module} );
 
         # Return result
         return $modules{$module};
@@ -142,7 +154,7 @@ Paranoid::Module -- Paranoid Module Loading Routines
 
 =head1 VERSION
 
-$Id: lib/Paranoid/Module.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+$Id: lib/Paranoid/Module.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -157,6 +169,18 @@ modules at runtime, along with importation of the exported symbol table.
 Specific functions and/or tag sets can be declared, just as you would in a
 normal B<use> or B<import> statement.
 
+=head1 IMPORT LISTS
+
+This module exports the following symbols by default:
+
+    loadModule
+
+The following specialized import lists also exist:
+
+    List        Members
+    --------------------------------------------------------
+    all         @defaults
+
 =head1 SUBROUTINES/METHODS
 
 =head2 loadModule
@@ -205,8 +229,22 @@ Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is licensed under the same terms as Perl, itself. 
-Please see http://dev.perl.org/licenses/ for more information.
+This software is free software.  Similar to Perl, you can redistribute it
+and/or modify it under the terms of either:
 
-(c) 2005 - 2017, Arthur Corliss (corliss@digitalmages.com)
+  a)     the GNU General Public License
+         <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+         Free Software Foundation <http://www.fsf.org/>; either version 1
+         <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+         <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+  b)     the Artistic License 2.0
+         <https://opensource.org/licenses/Artistic-2.0>,
+
+subject to the following additional term:  No trademark rights to
+"Paranoid" have been or are conveyed under any of the above licenses.
+However, "Paranoid" may be used fairly to describe this unmodified
+software, in good faith, but not as a trademark.
+
+(c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+(tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 
