@@ -1,11 +1,25 @@
 # Paranoid::Network -- Network functions for paranoid programs
 #
-# (c) 2005 - 2017, Arthur Corliss <corliss@digitalmages.com>
+# $Id: lib/Paranoid/Network.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 #
-# $Id: lib/Paranoid/Network.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+# This software is free software.  Similar to Perl, you can redistribute it
+# and/or modify it under the terms of either:
 #
-#    This software is licensed under the same terms as Perl, itself.
-#    Please see http://dev.perl.org/licenses/ for more information.
+#   a)     the GNU General Public License
+#          <https://www.gnu.org/licenses/gpl-1.0.html> as published by the
+#          Free Software Foundation <http://www.fsf.org/>; either version 1
+#          <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+#          <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+#   b)     the Artistic License 2.0
+#          <https://opensource.org/licenses/Artistic-2.0>,
+#
+# subject to the following additional term:  No trademark rights to
+# "Paranoid" have been or are conveyed under any of the above licenses.
+# However, "Paranoid" may be used fairly to describe this unmodified
+# software, in good faith, but not as a trademark.
+#
+# (c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+# (tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 #
 #####################################################################
 
@@ -29,7 +43,7 @@ use Paranoid::Network::Socket;
 use Paranoid::Network::IPv4 qw(:all);
 use Paranoid::Network::IPv6 qw(:all);
 
-($VERSION) = ( q$Revision: 2.07 $ =~ /(\d+(?:\.\d+)+)/sm );
+($VERSION) = ( q$Revision: 2.10 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(ipInNetworks hostInDomains extractIPs netIntersect);
 @EXPORT_OK   = ( @EXPORT, qw(NETMATCH HOSTNAME_REGEX) );
@@ -66,8 +80,7 @@ sub ipInNetworks {
     my $rv       = 0;
     my ( $family, @tmp );
 
-    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $ip, @networks );
-    pIn();
+    subPreamble( PDLEVEL1, '$@', $ip, @networks );
 
     NETMATCH = undef;
 
@@ -118,8 +131,7 @@ sub ipInNetworks {
         }
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -135,8 +147,7 @@ sub hostInDomains {
     my $rv      = 0;
     my $domain;
 
-    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $host, @domains );
-    pIn();
+    subPreamble( PDLEVEL1, '$@', $host, @domains );
 
     NETMATCH = undef;
 
@@ -158,8 +169,7 @@ sub hostInDomains {
         }
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -173,8 +183,7 @@ sub extractIPs {
     my @strings = @_;
     my ( $string, @ips, $ip, @tmp, @rv );
 
-    pdebug( 'entering w/%d strings', PDLEVEL1, scalar @strings );
-    pIn();
+    subPreamble( PDLEVEL1, '@', @strings );
 
     foreach $string (@strings) {
         next unless defined $string;
@@ -192,8 +201,8 @@ sub extractIPs {
         # for IPv6 addresses
         if ( has_ipv6() or $] >= 5.012 ) {
 
-            @ips = ( $string =~
-                    m/(@{[ IPV6CIDRRGX ]}|@{[ IPV6REGEX ]})/sogix );
+            @ips =
+                ( $string =~ m/(@{[ IPV6CIDRRGX ]}|@{[ IPV6REGEX ]})/sogix );
 
             # Filter out addresses with more than one ::
             @ips = grep { scalar(m/(::)/sg) <= 1 } @ips;
@@ -207,8 +216,7 @@ sub extractIPs {
         }
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, @rv );
+    subPostamble( PDLEVEL1, '@', @rv );
 
     return @rv;
 }
@@ -229,8 +237,7 @@ sub netIntersect {
     my $dest   = shift;
     my $rv     = 0;
 
-    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $target, $dest );
-    pIn();
+    subPreamble( PDLEVEL1, '$$', $target, $dest );
 
     if ( defined $target and defined $dest ) {
         if ( $target =~ m/^(?:@{[ IPV4CIDRRGX ]}|@{[ IPV4REGEX ]})$/s ) {
@@ -251,8 +258,7 @@ sub netIntersect {
         pdebug( 'one or both arguments are not defined', PDLEVEL1 );
     }
 
-    pOut();
-    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+    subPostamble( PDLEVEL1, '$', $rv );
 
     return $rv;
 }
@@ -267,7 +273,7 @@ Paranoid::Network - Network functions for paranoid programs
 
 =head1 VERSION
 
-$Id: lib/Paranoid/Network.pm, 2.07 2019/01/30 18:25:27 acorliss Exp $
+$Id: lib/Paranoid/Network.pm, 2.10 2022/03/08 00:01:04 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -289,6 +295,18 @@ caveats.  Please consult L<Paranoid::Network::Socket> for more details.
 
 I<NETMATCH> and I<HOSTNAME_REGEX> are not exported by default.
 
+=head1 IMPORT LISTS
+
+This module exports the following symbols by default:
+
+    ipInNetworks hostInDomains extractIPs netIntersect
+
+The following specialized import lists also exist:
+
+    List        Members
+    --------------------------------------------------------
+    all         @defaults NETMATCH HOSTNAME_REGEX
+
 =head1 SUBROUTINES/METHODS
 
 =head2 ipInNetworks
@@ -427,8 +445,22 @@ Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is licensed under the same terms as Perl, itself. 
-Please see http://dev.perl.org/licenses/ for more information.
+This software is free software.  Similar to Perl, you can redistribute it
+and/or modify it under the terms of either:
 
-(c) 2005 - 2017, Arthur Corliss (corliss@digitalmages.com)
+  a)     the GNU General Public License
+         <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+         Free Software Foundation <http://www.fsf.org/>; either version 1
+         <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+         <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+  b)     the Artistic License 2.0
+         <https://opensource.org/licenses/Artistic-2.0>,
+
+subject to the following additional term:  No trademark rights to
+"Paranoid" have been or are conveyed under any of the above licenses.
+However, "Paranoid" may be used fairly to describe this unmodified
+software, in good faith, but not as a trademark.
+
+(c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+(tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 
